@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2007-2018, Arvid Norberg
+Copyright (c) 2007-2008, 2010-2011, 2013-2019, 2022, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -93,12 +93,12 @@ extern TORRENT_EXPORT char const* libtorrent_assert_log;
 
 #if TORRENT_USE_IOSTREAM
 #define TORRENT_ASSERT_VAL(x, y) \
-	do { if (x) {} else { std::stringstream _s_; _s_ << #y ": " << y; \
-	libtorrent::assert_fail(#x, __LINE__, __FILE__, __func__, _s_.str().c_str(), 0); } } TORRENT_WHILE_0
+	do { if (x) {} else { std::stringstream _s; _s << #y ": " << y; \
+	libtorrent::assert_fail(#x, __LINE__, __FILE__, __func__, _s.str().c_str(), 0); } } TORRENT_WHILE_0
 
 #define TORRENT_ASSERT_FAIL_VAL(y) \
-	do { std::stringstream _s_; _s_ << #y ": " << y; \
-	libtorrent::assert_fail("<unconditional>", __LINE__, __FILE__, __func__, _s_.str().c_str(), 0); } TORRENT_WHILE_0
+	do { std::stringstream _s; _s << #y ": " << y; \
+	libtorrent::assert_fail("<unconditional>", __LINE__, __FILE__, __func__, _s.str().c_str(), 0); } TORRENT_WHILE_0
 
 #else
 #define TORRENT_ASSERT_VAL(x, y) TORRENT_ASSERT(x)
